(*
  Bifton - A Pascal- compiler.

  Copyright (C) 2004 Samuel Mimram & Julien Cristau.

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*)

(* $Id: bifton.ml,v 1.13 2004/05/03 15:31:01 smimou Exp $ *)

let usage = "usage: bifton [options] source"

let filename = ref ""
let out_fname = ref "out"

let show_ast1 = ref false
let show_ast2 = ref false

let arch = ref "i386"

let () =
  Printf.printf "Bifton 0.1.0 - A Pascal- compiler\n\nCopyright (C) 2004 by Samuel Mimram & Julien Cristau\nCompiled with OCaml %s\n\n%!" Sys.ocaml_version;
  Arg.parse
    [
      "-a", Arg.Set_string arch, "     Destination architecture (i386 or sparc)";
      "-o", Arg.Set_string out_fname, "     Output file name";
      "-s1", Arg.Set show_ast1, "     Show first AST (the Pascal-like one)";
      "-s2", Arg.Set show_ast2, "     Show second AST (the asm-like one)";
    ]
    (fun s -> filename := s) usage;
  if !filename = "" then (Printf.printf "%s\n\n" usage; exit 1);
  let fi =
    try
      open_in !filename
    with
      | Sys_error _ ->
	  Log.error "Source file not found"; exit 2
  in
  let parse_error s (l, c) =
    Log.error (Printf.sprintf "Syntax error line %d, column %d: %s" l c s); exit 3
  in
  let prog =
    try
      Parser.program Lexer.token (Lexing.from_channel fi)
    with
      | Pascal_ir.Parse_error(s, p) -> parse_error s p
      | Failure "lexing: empty token" 
      | Parsing.Parse_error ->
	  Log.error "Syntaxically incorrect program"; exit 3
  in
    close_in fi;
    Log.information "Source code sucessfully parsed";
    (
      try
	Pascal_ir.check prog
      with
	| Pascal_ir.Parse_error(s, p) -> parse_error s p
    );
    if !show_ast1 then
      (
	Log.information "Showing first AST";
	Printf.printf "\n%s\n" (Pascal_ir.pretty_print prog)
      );
    let p = Pascal_ir.asm_of_prog prog in
      Log.information "Asm IR sucessfully generated";
      p.Asm_ir.prog_source_name <- !filename;
      let gen_elf = String.sub !out_fname ((String.length !out_fname) - 2) 2 <> ".s" in
      let oc = open_out (!out_fname ^ (if gen_elf then ".s" else "")) in
      let asm =
	match !arch with
	  | "i386" -> I386.generate_code p
	  | "sparc" -> Sparc.generate_code p
	  | _ -> Log.error "Unknown target architecture"; exit 1
      in
	Printf.fprintf oc "%s" asm;
	close_out oc;
	Log.information "Assembly successfully generated";
	if gen_elf then
	  match (Unix.system ("/usr/bin/gcc -Wall -g glibb.o " ^ !out_fname ^ ".s" ^ " -o " ^ !out_fname)) with
	    | Unix.WEXITED 0 -> Log.information "Program successfully compiled :)";
	    | _ -> exit 1

