{
  (*
    Bifton - A Pascal- compiler.

    Copyright (C) 2004 Samuel Mimram & Julien Cristau.

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  *)

  (**
    Pascal- lexer.

    @author Samuel Mimram
  *)

  (* $Id: lexer.mll,v 1.9 2004/04/29 11:18:57 smimou Exp $ *)

  open Lexing
  open Parser

  let comment_depth = ref 0

  let on_newline lexbuf =
    lexbuf.lex_curr_p <-
    {
      pos_fname = lexbuf.lex_curr_p.pos_fname;
      pos_lnum = lexbuf.lex_curr_p.pos_lnum + 1;
      pos_bol = lexbuf.lex_curr_p.pos_cnum;
      pos_cnum = lexbuf.lex_curr_p.pos_cnum
    }
}

let space = ' ' | '\t' | '\r'

rule token = parse
  | "program" { PROGRAM }
  | "const" { CONST }
  | "type" { TYPE }
  | "array" { ARRAY }
  | "of" { OF }
  | "record" { RECORD }
  | "end" { END }
  | "var" { VAR }
  | "if" { IF }
  | "then" { THEN }
  | "else" { ELSE }
  | "while" { WHILE }
  | "do" { DO }
  | "begin" { BEGIN }
  | "end" { END }
  | "procedure" { PROCEDURE }
  | "function" { FUNCTION }
  | "return" { RETURN }
  | "new" { NEW }
  | "delete" { DELETE }

  | "true" { TRUE }
  | "false" { FALSE }
  | "not" { NOT }

  | "integer" { INTEGER }
  | "boolean" { BOOLEAN }
  | "^" { POINTER }
  | "&" { ADDRESS_OF }

  | "[" { LBRACK }
  | "]" { RBRACK }
  | ".." { INTERV }
  | ";" { SEMICOL }
  | "," { COMMA }
  | ":" { COLON }
  | "." { DOT }
  | "(" { LPAR }
  | ")" { RPAR }
  | ":=" { ASSIGN }

  | "=" { EQ }
  | "<>" { NEQ }
  | "<=" { LE }
  | "<" { LT }
  | ">=" { GE }
  | ">" { GT }

  | "+" { PLUS }
  | "-" { MINUS }
  | "or" { OR }
  | "*" { MULT }
  | "/" { DIV }
  | "mod" { MOD }
  | "and" { AND }

  | ['0'-'9']+ { NUMERAL (int_of_string (lexeme lexbuf)) }
  | ['a'-'z''A'-'Z''0'-'9']['a'-'z''A'-'Z''0'-'9''_']*['\'']? { NAME (lexeme lexbuf) }

  | space+ { token lexbuf }

  (** We have to manually increase the line number! Why? *)
  | '\n' { on_newline lexbuf; token lexbuf }

  | "{" { comment_depth := 1; in_comment lexbuf }

and in_comment = parse
  | "{" { incr comment_depth; in_comment lexbuf }
  | "}" { decr comment_depth; (if !comment_depth = 0 then token else in_comment) lexbuf }
  | '\n' { on_newline lexbuf; in_comment lexbuf }
  | [^'{''\n''}']+ { in_comment lexbuf }

{
}
